c
c	This file has the modifications to the main program needed to
c	computing the radial distribution function (new stuff below is 
c	in CAPITAL letters), and a new subroutine rdf1.  The other
c	subroutines in mc_nvt.f must be combined with this.
c
      program mc_nvt
c                                                    
c   equation of state of the lennard-jones fluid     
c                                                    

c   data files:
c	fort.10 - input  - job parameters
c	fort.11 - input  - initial atomic positions (if available)
c	fort.21 - output - interim atomic positions during run, and 
c                          final positons at the end 
c	fort.22 - output - sequence of atomic positions in pdb format
c	fort.30 - output - instantaneous energy and pressure

      implicit none
      include 'params.inc'
      integer equil, prod, nsamp, ii, icycl, ndispl, attempt, 
     &        nacc, ncycl, nmoves, imove, kkk
      real en, ent, vir, virt, dr, av1, av2, press
 
      INTEGER I,NBINS,NR
      REAL PI, RDEL, RVOL, VI
      PARAMETER (NBINS=1000)
      INTEGER IDIST(NBINS)
      REAL BIN(NBINS),VOLI(NBINS)

      write (6, *) 
      write (6, *) '**************** mc_nvt ***************'

c     ---initialize system

      call readdat(equil, prod, nsamp, ndispl, dr)

C     PARAMETERS FOR SAMPLING G(R)
C
      RDEL=0.025
      NR=HBOX/RDEL
      IF(NR.GT.NBINS) THEN
          WRITE(6,*) 'RDF BIN ARRAY TOO SMALL, NR =',Nr
          STOP
      ENDIF
      VI=2.*BOX**3/(PROD*NPART**2)
      PI=4.*ATAN(1.0)
      DO I=1,NR
        RVOL=4.*PI*RDEL**3*(I**2-I+1./3.)
        VOLI(I)=VI/RVOL
        BIN(I)=(I-0.5)*RDEL
      ENDDO

      IDIST = 0

      nmoves = ndispl
      av1    = 0.0
      av2    = 0.0

c     ---total energy of the system

      call toterg(en, vir)
      write (6, 99001) en, vir

c     ---start mc-cycle

      do ii = 1, 2

c        --- ii=1 equilibration
c        --- ii=2 production

         if (ii.eq.1) then
            ncycl = equil
            if (ncycl.ne.0) write (6, *) ' start equilibration '
         else
            if (ncycl.ne.0) write (6, *) ' start production '
            ncycl = prod
         end if
        
         attempt = 0
         nacc    = 0

c        ---intialize the subroutine that adjust the maximum displacement

         call adjust(attempt, nacc, dr)

         do icycl = 1, ncycl

            do imove = 1, nmoves

c              ---attempt to displace a particle

               call mcmove(en, vir, attempt, nacc, dr)
            end do

            if (ii.eq.2) then

C	       ---INSTANTANEOUS RDF 

	       CALL RDF1(IDIST,RDEL)

c              ---instantaneous sample averages

               if (mod(icycl,nsamp).eq.0) then
                  call sample(icycl, en, vir, press)
                  av1 = av1 + press
                  av2 = av2 + 1.0

               endif
            endif

c           ---write output in pdb format

c              if(mod(icycl,20).eq.0) call writepdb(21)

            if (mod(icycl,ncycl/5).eq.0) then
               write (6, *) '======>> done ', icycl, ' out of ', ncycl

c              ---write intermediate configuration to file

               call store(20, dr)

c              ---adjust maximum displacements

               call adjust(attempt, nacc, dr)
            end if
         end do
         if (ncycl.ne.0) then
            if (attempt.ne.0) write (6, 99003) attempt, nacc, 
     &                               100.0*real(nacc)/real(attempt)

c           ---test total energy

            call toterg(ent, virt)
            if (abs(ent-en).gt.1.e-6) then
               write (6, *) 
     &                    ' ######### problems energy ################ '
            end if
            if (abs(virt-vir).gt.1.e-6) then
               write (6, *) 
     &              ' ######### problems virial ################ '
            end if
            write (6, 99002) ent, en, ent - en, virt, vir, virt - vir
            write (6,*)

c     print chemical potential and pressure   

            if(ii.eq.2) then
               write (6,*) 'average pressure                  : ',
     &              av1/av2
            end if
         end if
      end do

C
C     OUTPUT RDF
C
      	WRITE(40,*) 'RDF OUTPUT'

       DO I=1,NR
            WRITE(40,'(F8.4,E16.7)') BIN(I),IDIST(I)*VOLI(I)
       ENDDO
C
     
      stop
 
99001 format (' total energy initial configuration: ', f12.5, /, 
     &        ' total virial initial configuration: ', f12.5,/)
99002 format (/,' total energy end of simulation    : ', f12.5, /, 
     &        '       running energy              : ', f12.5, /, 
     &        '       difference                  :  ', e12.5, /, 
     &        ' total virial end of simulation    : ', f12.5, /, 
     &        '       running virial              : ', f12.5, /, 
     &        '       difference                  :  ', e12.5)
99003 format (' number of att. to displ. a part.  : ', i10, /, 
     &        ' success: ', i10, '(= ', f5.2, '%)')
      end



      subroutine rdf1(idist,rdel)
c
c      computes instantaneous rdf histogram
c
      implicit none
      include 'params.inc'
      integer i,j,ij,nbins
      parameter (nbins=1000)
      integer idist(nbins)
      real xi,yi,zi,dx,dy,dz,r2,rdel

      do i=1,npart-1
        xi = x(i)
        yi = y(i)
        zi = z(i)
        do j=i+1,npart
            dx = xi - x(j)
            dy = yi - y(j)
            dz = zi - z(j)
            if (dx.gt.hbox) then
               dx = dx - box
            elseif (dx.lt.-hbox) then
               dx = dx + box
            end if
            if (dy.gt.hbox) then
               dy = dy - box
            elseif (dy.lt.-hbox) then
               dy = dy + box
            end if
            if (dz.gt.hbox) then
               dz = dz - box
            elseif (dz.lt.-hbox) then
               dz = dz + box
            end if
            r2 = dx*dx + dy*dy + dz*dz
            ij=sqrt(r2)/rdel+1.0
            idist(ij)=idist(ij)+1
        enddo
      enddo

      return
      end
